/*
  xdrv_42_irmp.ino - complete integration of IRMP for Tasmota

  Copyright (C) 2020  Erik Kunze <tasmota@erik-kunze.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef USE_IRMP
/*********************************************************************************************\
 * IR Remote send and receive using IRMP library
\*********************************************************************************************/

#define XDRV_42             42

#define IRMP_PROTOCOL_NAMES 1
#define IRMP_IRSND_ALLOW_DYNAMIC_PINS 1
#define F_INTERRUPTS 20000 // Instead of default 15000 to support LEGO + RCMM protocols
#define USE_ONE_TIMER_FOR_IRMP_AND_IRSND 1
#include <irmp.c.h>
#include <irsnd.c.h>
#include <irmpSelectAllProtocols.h>  // This enables all possible protocols


/*********************************************************************************************\
 * IR Send
\*********************************************************************************************/

void IrSendInit(void)
{
  AddLog_Debug(PSTR("IRMP: pre isnd_init(%d)"), Pin(GPIO_IRSEND));
  irsnd_init(Pin(GPIO_IRSEND)); // an IR led is at GPIO_IRSEND
  AddLog_Debug(PSTR("IRMP: post isnd_init(%d)"), Pin(GPIO_IRSEND));
}

/*********************************************************************************************\
 * IR Receive
\*********************************************************************************************/

void IrReceiveInit(void)
{
  AddLog_Debug(PSTR("IRMP: pre irmp_init(%d)"), Pin(GPIO_IRRECV));
  irmp_init(Pin(GPIO_IRRECV));
  AddLog_Debug(PSTR("IRMP: post irmp_init(%d)"), Pin(GPIO_IRRECV));
}

/*********************************************************************************************\
 * Interface
\*********************************************************************************************/

bool Xdrv42(uint8_t function)
{
  bool result = false;
  if (PinUsed(GPIO_IRSEND) || PinUsed(GPIO_IRRECV)) {
    switch (function) {
      case FUNC_PRE_INIT:
        if (PinUsed(GPIO_IRSEND)) {
          IrSendInit();
        }
        if (PinUsed(GPIO_IRRECV)) {
          IrReceiveInit();
        }
        break;
      case FUNC_EVERY_50_MSECOND:
        if (PinUsed(GPIO_IRRECV)) {
//          IrReceiveCheck();  // check if there's anything on IR side
        }
        break;
    }
  }
  return result;
}

#endif // USE_IRMP
